@@ -16,17 +16,22 @@
 
 use std::rc::Rc;
 
+#[allow(unused_imports)]
+use ipnet;
+
 use super::Result;
 use super::auth::{self, AuthMethod};
 #[allow(unused_imports)]
-use super::common::FlavorRef;
+use super::common::{FlavorRef, NetworkRef};
 #[cfg(feature = "compute")]
 use super::compute::{Flavor, FlavorQuery, FlavorSummary, KeyPair, KeyPairQuery,
                      NewKeyPair, NewServer, Server, ServerQuery, ServerSummary};
 #[cfg(feature = "image")]
 use super::image::{Image, ImageQuery};
 #[cfg(feature = "network")]
-use super::network::{Network, NetworkQuery};
+use super::network::{FloatingIp, FloatingIpQuery, Network, NetworkQuery,
+                     NewFloatingIp, NewNetwork, NewPort, NewSubnet, Port,
+                     PortQuery, Subnet, SubnetQuery};
 use super::session::Session;
 
 
@@ -47,21 +52,44 @@ impl Cloud {
     /// # Example
     ///
     /// ```rust,no_run
-    /// fn cloud_from_env() -> openstack::Result<openstack::Cloud> {
-    ///     openstack::auth::from_env().map(openstack::Cloud::new)
+    /// fn cloud() -> openstack::Result<openstack::Cloud> {
+    ///     let auth = openstack::auth::Password::new(
+    ///             "https://cloud.example.com",
+    ///             "user1", "pa$$word", "Default")
+    ///         .expect("Invalid authentication URL")
+    ///         .with_project_scope("project1", "Default");
+    ///     Ok(openstack::Cloud::new(auth))
     /// }
     ///
-    /// # fn main() { cloud_from_env().unwrap(); }
+    /// # fn main() { cloud().unwrap(); }
     /// ```
     ///
-    /// Note: in this particular case it's better to use
-    /// [from_env](#method.from_env).
+    /// # See Also
+    ///
+    /// * [from_config](#method.from_config) to create a Cloud from clouds.yaml
+    /// * [from_env](#method.from_env) to create a Cloud from environment variables
     pub fn new<Auth: AuthMethod + 'static>(auth_method: Auth) -> Cloud {
         Cloud {
             session: Rc::new(Session::new(auth_method))
         }
     }
 
+    /// Create a new cloud object from a configuration file
+    ///
+    /// # Example
+    ///
+    /// ```rust,no_run
+    /// # fn cloud_from_config() -> openstack::Result<()> {
+    /// let os = openstack::Cloud::from_config("cloud-1")?;
+    /// # Ok(()) }
+    /// # fn main() { cloud_from_config().unwrap(); }
+    /// ```
+    pub fn from_config<S: AsRef<str>>(cloud_name: S) -> Result<Cloud> {
+        Ok(Cloud {
+            session: Rc::new(auth::from_config(cloud_name)?)
+        })
+    }
+
     /// Create a new cloud object from environment variables.
     ///
     /// # Example
@@ -74,7 +102,7 @@ impl Cloud {
     /// ```
     pub fn from_env() -> Result<Cloud> {
         Ok(Cloud {
-            session: Rc::new(Session::new(auth::from_env()?))
+            session: Rc::new(auth::from_env()?)
         })
     }
 
@@ -110,6 +138,15 @@ impl Cloud {
         FlavorQuery::new(self.session.clone())
     }
 
+    /// Build a query against floating IP list.
+    ///
+    /// The returned object is a builder that should be used to construct
+    /// the query.
+    #[cfg(feature = "network")]
+    pub fn find_floating_ips(&self) -> FloatingIpQuery {
+        FloatingIpQuery::new(self.session.clone())
+    }
+
     /// Build a query against image list.
     ///
     /// The returned object is a builder that should be used to construct
@@ -137,6 +174,15 @@ impl Cloud {
         NetworkQuery::new(self.session.clone())
     }
 
+    /// Build a query against port list.
+    ///
+    /// The returned object is a builder that should be used to construct
+    /// the query.
+    #[cfg(feature = "network")]
+    pub fn find_ports(&self) -> PortQuery {
+        PortQuery::new(self.session.clone())
+    }
+
     /// Build a query against server list.
     ///
     /// The returned object is a builder that should be used to construct
@@ -160,6 +206,15 @@ impl Cloud {
         ServerQuery::new(self.session.clone())
     }
 
+    /// Build a query against subnet list.
+    ///
+    /// The returned object is a builder that should be used to construct
+    /// the query.
+    #[cfg(feature = "network")]
+    pub fn find_subnets(&self) -> SubnetQuery {
+        SubnetQuery::new(self.session.clone())
+    }
+
     /// Find a flavor by its name or ID.
     ///
     /// # Example
@@ -172,7 +227,23 @@ impl Cloud {
     /// ```
     #[cfg(feature = "compute")]
     pub fn get_flavor<Id: AsRef<str>>(&self, id_or_name: Id) -> Result<Flavor> {
-        Flavor::new(self.session.clone(), id_or_name)
+        Flavor::load(self.session.clone(), id_or_name)
+    }
+
+    /// Find a floating IP by its ID.
+    ///
+    /// # Example
+    ///
+    /// ```rust,no_run
+    /// use openstack;
+    ///
+    /// let os = openstack::Cloud::from_env().expect("Unable to authenticate");
+    /// let server = os.get_floating_ip("031e08c7-2ca7-4c0b-9923-030c8d946ba4")
+    ///     .expect("Unable to get a floating IP");
+    /// ```
+    #[cfg(feature = "network")]
+    pub fn get_floating_ip<Id: AsRef<str>>(&self, id: Id) -> Result<FloatingIp> {
+        FloatingIp::load(self.session.clone(), id)
     }
 
     /// Find an image by its name or ID.
@@ -217,7 +288,23 @@ impl Cloud {
     /// ```
     #[cfg(feature = "network")]
     pub fn get_network<Id: AsRef<str>>(&self, id_or_name: Id) -> Result<Network> {
-        Network::new(self.session.clone(), id_or_name)
+        Network::load(self.session.clone(), id_or_name)
+    }
+
+    /// Find an port by its name or ID.
+    ///
+    /// # Example
+    ///
+    /// ```rust,no_run
+    /// use openstack;
+    ///
+    /// let os = openstack::Cloud::from_env().expect("Unable to authenticate");
+    /// let server = os.get_port("4d9c1710-fa02-49f9-8218-291024ef4140")
+    ///     .expect("Unable to get a port");
+    /// ```
+    #[cfg(feature = "network")]
+    pub fn get_port<Id: AsRef<str>>(&self, id_or_name: Id) -> Result<Port> {
+        Port::load(self.session.clone(), id_or_name)
     }
 
     /// Find a server by its name or ID.
@@ -233,7 +320,23 @@ impl Cloud {
     /// ```
     #[cfg(feature = "compute")]
     pub fn get_server<Id: AsRef<str>>(&self, id_or_name: Id) -> Result<Server> {
-        Server::new(self.session.clone(), id_or_name)
+        Server::load(self.session.clone(), id_or_name)
+    }
+
+    /// Find an subnet by its name or ID.
+    ///
+    /// # Example
+    ///
+    /// ```rust,no_run
+    /// use openstack;
+    ///
+    /// let os = openstack::Cloud::from_env().expect("Unable to authenticate");
+    /// let server = os.get_subnet("private-subnet")
+    ///     .expect("Unable to get a subnet");
+    /// ```
+    #[cfg(feature = "network")]
+    pub fn get_subnet<Id: AsRef<str>>(&self, id_or_name: Id) -> Result<Subnet> {
+        Subnet::load(self.session.clone(), id_or_name)
     }
 
     /// List all flavors.
@@ -255,6 +358,25 @@ impl Cloud {
         self.find_flavors().all()
     }
 
+    /// List all floating IPs
+    ///
+    /// This call can yield a lot of results, use the
+    /// [find_floating_ips](#method.find_floating_ips) call to limit the number of
+    /// networks to receive.
+    ///
+    /// # Example
+    ///
+    /// ```rust,no_run
+    /// use openstack;
+    ///
+    /// let os = openstack::Cloud::from_env().expect("Unable to authenticate");
+    /// let server_list = os.list_floating_ips().expect("Unable to fetch floating IPs");
+    /// ```
+    #[cfg(feature = "network")]
+    pub fn list_floating_ips(&self) -> Result<Vec<FloatingIp>> {
+        self.find_floating_ips().all()
+    }
+
     /// List all images.
     ///
     /// This call can yield a lot of results, use the
@@ -308,6 +430,25 @@ impl Cloud {
         self.find_networks().all()
     }
 
+    /// List all ports.
+    ///
+    /// This call can yield a lot of results, use the
+    /// [find_ports](#method.find_ports) call to limit the number of
+    /// ports to receive.
+    ///
+    /// # Example
+    ///
+    /// ```rust,no_run
+    /// use openstack;
+    ///
+    /// let os = openstack::Cloud::from_env().expect("Unable to authenticate");
+    /// let server_list = os.list_ports().expect("Unable to fetch ports");
+    /// ```
+    #[cfg(feature = "network")]
+    pub fn list_ports(&self) -> Result<Vec<Port>> {
+        self.find_ports().all()
+    }
+
     /// List all servers.
     ///
     /// This call can yield a lot of results, use the
@@ -327,6 +468,35 @@ impl Cloud {
         self.find_servers().all()
     }
 
+    /// List all subnets.
+    ///
+    /// This call can yield a lot of results, use the
+    /// [find_subnets](#method.find_subnets) call to limit the number of
+    /// subnets to receive.
+    ///
+    /// # Example
+    ///
+    /// ```rust,no_run
+    /// use openstack;
+    ///
+    /// let os = openstack::Cloud::from_env().expect("Unable to authenticate");
+    /// let server_list = os.list_subnets().expect("Unable to fetch subnets");
+    /// ```
+    #[cfg(feature = "network")]
+    pub fn list_subnets(&self) -> Result<Vec<Subnet>> {
+        self.find_subnets().all()
+    }
+
+    /// Prepare a new floating IP for creation.
+    ///
+    /// This call returns a `NewFloatingIp` object, which is a builder
+    /// to populate floating IP fields.
+    #[cfg(feature = "network")]
+    pub fn new_floating_ip<N>(&self, floating_network: N) -> NewFloatingIp
+            where N: Into<NetworkRef> {
+        NewFloatingIp::new(self.session.clone(), floating_network.into())
+    }
+
     /// Prepare a new key pair for creation.
     ///
     /// This call returns a `NewKeyPair` object, which is a builder to populate
@@ -336,6 +506,24 @@ impl Cloud {
         NewKeyPair::new(self.session.clone(), name.into())
     }
 
+    /// Prepare a new network for creation.
+    ///
+    /// This call returns a `NewNetwork` object, which is a builder to populate
+    /// network fields.
+    #[cfg(feature = "network")]
+    pub fn new_network(&self) -> NewNetwork {
+        NewNetwork::new(self.session.clone())
+    }
+
+    /// Prepare a new port for creation.
+    ///
+    /// This call returns a `NewPort` object, which is a builder to populate
+    /// port fields.
+    #[cfg(feature = "network")]
+    pub fn new_port<N>(&self, network: N) -> NewPort where N: Into<NetworkRef> {
+        NewPort::new(self.session.clone(), network.into())
+    }
+
     /// Prepare a new server for creation.
     ///
     /// This call returns a `NewServer` object, which is a builder to populate
@@ -345,6 +533,31 @@ impl Cloud {
             where S: Into<String>, F: Into<FlavorRef> {
         NewServer::new(self.session.clone(), name.into(), flavor.into())
     }
+
+    /// Prepare a new subnet for creation.
+    ///
+    /// This call returns a `NewSubnet` object, which is a builder to populate
+    /// subnet fields.
+    ///
+    /// # Example
+    ///
+    /// ```rust,no_run
+    /// extern crate ipnet;
+    /// extern crate openstack;
+    /// use std::net;
+    ///
+    /// let os = openstack::Cloud::from_env().expect("Unable to authenticate");
+    /// let cidr = ipnet::Ipv4Net::new(net::Ipv4Addr::new(192, 168, 1, 0), 24)
+    ///     .unwrap().into();
+    /// let new_subnet = os.new_subnet("private-net", cidr)
+    ///     .with_name("private-subnet")
+    ///     .create().expect("Unable to create subnet");
+    /// ```
+    #[cfg(feature = "network")]
+    pub fn new_subnet<N>(&self, network: N, cidr: ipnet::IpNet) -> NewSubnet
+            where N: Into<NetworkRef> {
+        NewSubnet::new(self.session.clone(), network.into(), cidr)
+    }
 }
 
 
