@@ -14,39 +14,23 @@
 
 //! Types and traits shared between services.
 
-use std::rc::Rc;
-
-use serde::Serialize;
-
 use super::super::Result;
 use super::super::session::Session;
 
 
-/// Trait representing something that can be listed from a session.
-pub trait ListResources {
-    /// Default limit to use with this resource.
-    const DEFAULT_LIMIT: usize;
-
-    /// Whether pagination is supported for this resource.
-    fn can_paginate(_session: &Session) -> Result<bool> { Ok(true) }
-
-    /// List the resources from the session.
-    fn list_resources<Q>(session: Rc<Session>, query: Q) -> Result<Vec<Self>>
-        where Self: Sized, Q: Serialize + ::std::fmt::Debug;
-}
-
 /// Trait representing something that can be refreshed.
 pub trait Refresh {
     /// Refresh the resource representation.
     fn refresh(&mut self) -> Result<()>;
 }
 
-/// Trait representing something that has an ID.
-pub trait ResourceId {
-    /// Identifier of the current resource.
-    fn resource_id(&self) -> String;
+/// A type that can be converted into a verified representation.
+pub trait IntoVerified {
+    /// Conver this object into the same object with verification.
+    fn into_verified(self, session: &Session) -> Result<Self> where Self: Sized;
 }
 
+
 macro_rules! opaque_resource_type {
     ($(#[$attr:meta])* $name:ident ? $service:expr) => (
         $(#[$attr])*
@@ -80,6 +64,12 @@ macro_rules! opaque_resource_type {
             }
         }
 
+        impl From<$name> for ::serde_json::Value {
+            fn from(value: $name) -> ::serde_json::Value {
+                value.value.into()
+            }
+        }
+
         impl AsRef<str> for $name {
             fn as_ref(&self) -> &str {
                 &self.value
@@ -120,13 +110,14 @@ macro_rules! opaque_resource_type {
                     verified: true
                 }
             }
+        }
 
-            /// Verify this reference and convert to an ID, if possible.
-            #[cfg(not(feature = $service))]
-            #[allow(dead_code)]
-            pub(crate) fn into_verified(self, _session: &$crate::session::Session)
-                    -> $crate::Result<String> {
-                Ok(self.value)
+        #[cfg(not(feature = $service))]
+        #[allow(dead_code)]
+        impl $crate::common::IntoVerified for $name {
+            fn into_verified(self, _session: &$crate::session::Session)
+                    -> $crate::Result<$name> {
+                Ok(self)
             }
         }
     )
@@ -144,6 +135,10 @@ opaque_resource_type!(#[doc = "An ID of a `Project`"] ProjectRef ? "identity");
 
 opaque_resource_type!(#[doc = "An ID of a `Port`"] PortRef ? "network");
 
+opaque_resource_type!(#[doc = "An ID of a `Router`"] RouterRef ? "network");
+
+opaque_resource_type!(#[doc = "An ID of a `Subnet`"] SubnetRef ? "network");
+
 opaque_resource_type!(#[doc = "An ID of a `User`"] UserRef ? "identity");
 
 
