@@ -14,15 +14,14 @@
 
 //! Flavor management via Compute API.
 
-use std::fmt::Debug;
+use std::collections::HashMap;
 use std::rc::Rc;
 
 use fallible_iterator::{IntoFallibleIterator, FallibleIterator};
-use serde::Serialize;
 
 use super::super::{Error, Result};
-use super::super::common::{self, FlavorRef, ListResources, Refresh, ResourceId,
-                           ResourceIterator};
+use super::super::common::{self, FlavorRef, IntoVerified, Refresh,
+                           ResourceQuery, ResourceIterator};
 use super::super::session::Session;
 use super::super::utils::Query;
 use super::base::V2API;
@@ -33,7 +32,8 @@ use super::protocol;
 #[derive(Clone, Debug)]
 pub struct Flavor {
     session: Rc<Session>,
-    inner: protocol::Flavor
+    inner: protocol::Flavor,
+    extra_specs: HashMap<String, String>,
 }
 
 /// Structure representing a summary of a flavor.
@@ -43,7 +43,7 @@ pub struct FlavorSummary {
     inner: common::protocol::IdAndName,
 }
 
-/// A query to server list.
+/// A query to flavor list.
 #[derive(Clone, Debug)]
 pub struct FlavorQuery {
     session: Rc<Session>,
@@ -51,18 +51,36 @@ pub struct FlavorQuery {
     can_paginate: bool,
 }
 
+/// A detailed query to flavor list.
+#[derive(Clone, Debug)]
+pub struct DetailedFlavorQuery {
+    inner: FlavorQuery,
+}
+
 
 impl Flavor {
-    /// Load a Flavor object.
-    pub(crate) fn new<Id: AsRef<str>>(session: Rc<Session>, id: Id)
+    /// Create a flavor object.
+    pub(crate) fn new(session: Rc<Session>, mut inner: protocol::Flavor)
             -> Result<Flavor> {
-        let inner = session.get_flavor(id)?;
+        let extra_specs = match inner.extra_specs.take() {
+            Some(es) => es,
+            None => session.get_extra_specs_by_flavor_id(&inner.id)?
+        };
+
         Ok(Flavor {
             session: session,
-            inner: inner
+            inner: inner,
+            extra_specs: extra_specs,
         })
     }
 
+    /// Load a Flavor object.
+    pub(crate) fn load<Id: AsRef<str>>(session: Rc<Session>, id: Id)
+            -> Result<Flavor> {
+        let inner = session.get_flavor(id)?;
+        Flavor::new(session, inner)
+    }
+
     /// Get ephemeral disk size in GiB.
     ///
     /// Returns `0` when ephemeral disk was not requested.
@@ -70,6 +88,11 @@ impl Flavor {
         self.inner.ephemeral
     }
 
+    /// Extra specs of the flavor.
+    pub fn extra_specs(&self) -> &HashMap<String, String> {
+        &self.extra_specs
+    }
+
     /// Get a reference to flavor unique ID.
     pub fn id(&self) -> &String {
         &self.inner.id
@@ -111,7 +134,7 @@ impl Flavor {
 impl Refresh for Flavor {
     /// Refresh the flavor.
     fn refresh(&mut self) -> Result<()> {
-        self.inner = self.session.get_flavor(&self.inner.id)?;
+        self.inner = self.session.get_flavor_by_id(&self.inner.id)?;
         Ok(())
     }
 }
@@ -129,7 +152,7 @@ impl FlavorSummary {
 
     /// Get details.
     pub fn details(&self) -> Result<Flavor> {
-        Flavor::new(self.session.clone(), &self.inner.id)
+        Flavor::load(self.session.clone(), &self.inner.id)
     }
 }
 
@@ -160,6 +183,13 @@ impl FlavorQuery {
         self
     }
 
+    /// Convert this query into a detailed query.
+    pub fn detailed(self) -> DetailedFlavorQuery {
+        DetailedFlavorQuery {
+            inner: self
+        }
+    }
+
     /// Convert this query into an iterator executing the request.
     ///
     /// This iterator yields only `FlavorSummary` objects, containing
@@ -169,9 +199,9 @@ impl FlavorQuery {
     /// call returning a `Result`.
     ///
     /// Note that no requests are done until you start iterating.
-    pub fn into_iter(self) -> ResourceIterator<FlavorSummary> {
+    pub fn into_iter(self) -> ResourceIterator<FlavorQuery> {
         debug!("Fetching flavors with {:?}", self.query);
-        ResourceIterator::new(self.session, self.query)
+        ResourceIterator::new(self)
     }
 
     /// Convert this query into an iterator executing the request.
@@ -183,9 +213,9 @@ impl FlavorQuery {
     /// call returning a `Result`.
     ///
     /// Note that no requests are done until you start iterating.
-    pub fn into_iter_detailed(self) -> ResourceIterator<Flavor> {
-        debug!("Fetching flavor details with {:?}", self.query);
-        ResourceIterator::new(self.session, self.query)
+    #[deprecated(since = "0.2.0", note = "Use .detailed().into_iter()")]
+    pub fn into_iter_detailed(self) -> ResourceIterator<DetailedFlavorQuery> {
+        self.detailed().into_iter()
     }
 
     /// Execute this request and return all results.
@@ -211,40 +241,66 @@ impl FlavorQuery {
     }
 }
 
+impl ResourceQuery for FlavorQuery {
+    type Item = FlavorSummary;
 
-impl ResourceId for FlavorSummary {
-    fn resource_id(&self) -> String {
-        self.id().clone()
+    const DEFAULT_LIMIT: usize = 100;
+
+    fn can_paginate(&self) -> Result<bool> {
+        Ok(self.can_paginate)
     }
-}
 
-impl ListResources for FlavorSummary {
-    const DEFAULT_LIMIT: usize = 50;
+    fn extract_marker(&self, resource: &Self::Item) -> String {
+        resource.id().clone()
+    }
 
-    fn list_resources<Q: Serialize + Debug>(session: Rc<Session>, query: Q)
-            -> Result<Vec<FlavorSummary>> {
-        Ok(session.list_flavors(&query)?.into_iter().map(|item| FlavorSummary {
-            session: session.clone(),
+    fn fetch_chunk(&self, limit: Option<usize>, marker: Option<String>)
+            -> Result<Vec<Self::Item>> {
+        let query = self.query.with_marker_and_limit(limit, marker);
+        Ok(self.session.list_flavors(&query)?.into_iter().map(|item| FlavorSummary {
+            session: self.session.clone(),
             inner: item
         }).collect())
     }
 }
 
-impl ResourceId for Flavor {
-    fn resource_id(&self) -> String {
-        self.id().clone()
+impl DetailedFlavorQuery {
+    /// Convert this query into an iterator executing the request.
+    ///
+    /// This iterator yields full `Flavor` objects.
+    ///
+    /// Returns a `FallibleIterator`, which is an iterator with each `next`
+    /// call returning a `Result`.
+    ///
+    /// Note that no requests are done until you start iterating.
+    pub fn into_iter(self) -> ResourceIterator<DetailedFlavorQuery> {
+        debug!("Fetching detailed flavors with {:?}", self.inner.query);
+        ResourceIterator::new(self)
     }
 }
 
-impl ListResources for Flavor {
+impl ResourceQuery for DetailedFlavorQuery {
+    type Item = Flavor;
+
     const DEFAULT_LIMIT: usize = 50;
 
-    fn list_resources<Q: Serialize + Debug>(session: Rc<Session>, query: Q)
-            -> Result<Vec<Flavor>> {
-        Ok(session.list_flavors_detail(&query)?.into_iter().map(|item| Flavor {
-            session: session.clone(),
-            inner: item
-        }).collect())
+    fn can_paginate(&self) -> Result<bool> {
+        Ok(self.inner.can_paginate)
+    }
+
+    fn extract_marker(&self, resource: &Self::Item) -> String {
+        resource.id().clone()
+    }
+
+    fn fetch_chunk(&self, limit: Option<usize>, marker: Option<String>)
+            -> Result<Vec<Self::Item>> {
+        let query = self.inner.query.with_marker_and_limit(limit, marker);
+        let flavors = self.inner.session.list_flavors_detail(&query)?;
+        let mut result = Vec::with_capacity(flavors.len());
+        for item in flavors {
+            result.push(Flavor::new(self.inner.session.clone(), item)?);
+        }
+        Ok(result)
     }
 }
 
@@ -253,9 +309,21 @@ impl IntoFallibleIterator for FlavorQuery {
 
     type Error = Error;
 
-    type IntoIter = ResourceIterator<FlavorSummary>;
+    type IntoIter = ResourceIterator<FlavorQuery>;
 
-    fn into_fallible_iterator(self) -> ResourceIterator<FlavorSummary> {
+    fn into_fallible_iterator(self) -> Self::IntoIter {
+        self.into_iter()
+    }
+}
+
+impl IntoFallibleIterator for DetailedFlavorQuery {
+    type Item = Flavor;
+
+    type Error = Error;
+
+    type IntoIter = ResourceIterator<DetailedFlavorQuery>;
+
+    fn into_fallible_iterator(self) -> Self::IntoIter {
         self.into_iter()
     }
 }
@@ -272,14 +340,28 @@ impl From<FlavorSummary> for FlavorRef {
     }
 }
 
-impl FlavorRef {
+#[cfg(feature = "compute")]
+impl IntoVerified for FlavorRef {
     /// Verify this reference and convert to an ID, if possible.
-    #[cfg(feature = "compute")]
-    pub(crate) fn into_verified(self, session: &Session) -> Result<String> {
+    fn into_verified(self, session: &Session) -> Result<FlavorRef> {
         Ok(if self.verified {
-            self.value
+            self
         } else {
-            session.get_flavor(&self.value)?.id
+            FlavorRef::new_verified(session.get_flavor(&self.value)?.id)
         })
     }
 }
+
+impl From<Flavor> for protocol::ServerFlavor {
+    fn from(value: Flavor) -> protocol::ServerFlavor {
+        protocol::ServerFlavor {
+            ephemeral_size: value.inner.ephemeral,
+            extra_specs: Some(value.extra_specs),
+            original_name: value.inner.name,
+            ram_size: value.inner.ram,
+            root_size: value.inner.disk,
+            swap_size: value.inner.swap,
+            vcpu_count: value.inner.vcpus,
+        }
+    }
+}
