@@ -134,21 +134,57 @@ pub struct ServerAddress {
     pub addr_type: Option<AddressType>
 }
 
+#[derive(Clone, Debug, Deserialize)]
+pub struct ExtraSpecsRoot {
+    pub extra_specs: HashMap<String, String>
+}
+
+/// A summary information of a flavor used for a server.
+#[derive(Clone, Debug)]
+pub struct ServerFlavor {
+    /// Ephemeral disk size in GiB.
+    pub ephemeral_size: u64,
+    /// Extra specs (if present).
+    pub extra_specs: Option<HashMap<String, String>>,
+    /// Name of the original flavor.
+    pub original_name: String,
+    /// RAM size in MiB.
+    pub ram_size: u64,
+    /// Root disk size in GiB.
+    pub root_size: u64,
+    /// Swap disk size in MiB.
+    pub swap_size: u64,
+    /// VCPU count.
+    pub vcpu_count: u32,
+}
+
 #[derive(Clone, Debug, Deserialize)]
 pub struct Server {
-    #[serde(deserialize_with = "common::protocol::empty_as_none", default)]
-    pub accessIPv4: Option<Ipv4Addr>,
-    #[serde(deserialize_with = "common::protocol::empty_as_none", default)]
-    pub accessIPv6: Option<Ipv6Addr>,
+    #[serde(deserialize_with = "common::protocol::empty_as_none", default,
+            rename = "accessIPv4")]
+    pub access_ipv4: Option<Ipv4Addr>,
+    #[serde(deserialize_with = "common::protocol::empty_as_none", default,
+            rename = "accessIPv6")]
+    pub access_ipv6: Option<Ipv6Addr>,
     #[serde(default)]
     pub addresses: HashMap<String, Vec<ServerAddress>>,
     #[serde(rename = "OS-EXT-AZ:availability_zone")]
     pub availability_zone: String,
-    pub created: DateTime<FixedOffset>,
-    // TODO(dtantsur): flavor
+    #[serde(rename = "created")]
+    pub created_at: DateTime<FixedOffset>,
+    #[serde(deserialize_with = "common::protocol::empty_as_none", default)]
+    pub description: Option<String>,
+    // TODO(dtantsur): flavor in newer versions
+    pub flavor: common::protocol::Ref,
+    #[serde(deserialize_with = "common::protocol::empty_as_default",
+            rename = "config_drive")]
+    pub has_config_drive: bool,
     pub id: String,
     #[serde(deserialize_with = "common::protocol::empty_as_none", default)]
     pub image: Option<common::protocol::Ref>,
+    #[serde(rename = "key_name", deserialize_with = "common::protocol::empty_as_none",
+            default)]
+    pub key_pair_name: Option<String>,
     pub name: String,
     #[serde(default)]
     pub metadata: HashMap<String, String>,
@@ -156,7 +192,8 @@ pub struct Server {
     #[serde(rename = "OS-EXT-STS:power_state", default)]
     pub power_state: ServerPowerState,
     pub tenant_id: String,
-    pub updated: DateTime<FixedOffset>,
+    #[serde(rename = "updated")]
+    pub updated_at: DateTime<FixedOffset>,
     pub user_id: String
 }
 
@@ -210,7 +247,11 @@ pub struct CreatedServerRoot {
 pub struct Flavor {
     #[serde(rename = "OS-FLV-EXT-DATA:ephemeral", default)]
     pub ephemeral: u64,
+    #[serde(default, deserialize_with = "common::protocol::empty_as_none")]
+    pub description: Option<String>,
     pub disk: u64,
+    #[serde(default)]
+    pub extra_specs: Option<HashMap<String, String>>,
     pub id: String,
     #[serde(rename = "os-flavor-access:is_public",
             default = "default_flavor_is_public")]
