@@ -15,8 +15,17 @@
 //! Network API implementation bits.
 
 mod base;
+mod floatingips;
 mod networks;
+mod ports;
 mod protocol;
+mod subnets;
 
-pub use self::networks::{Network, NetworkQuery};
-pub use self::protocol::{NetworkStatus, NetworkSortKey};
+pub use self::floatingips::{FloatingIp, FloatingIpQuery, NewFloatingIp};
+pub use self::networks::{NewNetwork, Network, NetworkQuery};
+pub use self::ports::{NewPort, Port, PortIpAddress, PortIpRequest, PortQuery};
+pub use self::protocol::{AllocationPool, FloatingIpSortKey, FloatingIpStatus,
+                         HostRoute, Ipv6Mode, IpVersion, NetworkStatus,
+                         NetworkSortKey, PortExtraDhcpOption, PortForwarding,
+                         PortSortKey, SubnetSortKey};
+pub use self::subnets::{NewSubnet, Subnet, SubnetQuery};
