@@ -21,6 +21,8 @@ use std::collections::HashMap;
 use std::fmt;
 use std::hash::Hash;
 
+use serde::{Serialize, Serializer};
+
 use super::{Error, ErrorKind, Result};
 
 
@@ -60,6 +62,26 @@ impl Query {
             where K: Into<String>, V: Into<String> {
         self.0.push((param.into(), value.into()))
     }
+
+    /// Add marker and limit to the query and clone it.
+    pub fn with_marker_and_limit(&self, limit: Option<usize>,
+                                 marker: Option<String>) -> Query {
+        let mut new = self.clone();
+        if let Some(limit_) = limit {
+            new.push("limit", limit_);
+        }
+        if let Some(marker_) = marker {
+            new.push_str("marker", marker_);
+        }
+        new
+    }
+}
+
+impl Serialize for Query {
+    fn serialize<S>(&self, serializer: S) -> ::std::result::Result<S::Ok, S::Error>
+            where S: Serializer {
+        self.0.serialize(serializer)
+    }
 }
 
 impl<T: Clone> ValueCache<T> {
@@ -225,13 +247,10 @@ pub mod url {
 pub mod test {
     //! Common primitives for testing.
 
-    use reqwest::{IntoUrl, Url};
-    use reqwest::header::Headers;
+    use reqwest::IntoUrl;
 
-    use super::super::{Error, ErrorKind, Result};
-    use super::super::auth::{AuthMethod, NoAuth};
-    use super::super::common::ApiVersion;
-    use super::super::session::{Session, ServiceInfo, ServiceType};
+    use super::super::auth::NoAuth;
+    use super::super::session::Session;
 
     /// Create a session with fake authentication.
     pub fn new_session<U: IntoUrl>(endpoint: U) -> Session {
@@ -239,32 +258,5 @@ pub mod test {
         Session::new(auth)
     }
 
-    /// Fake service type.
-    pub struct FakeServiceType;
-
     pub const URL: &'static str = "https://127.0.0.1:5000/";
-
-    impl ServiceType for FakeServiceType {
-        fn catalog_type() -> &'static str { "fake" }
-
-        fn service_info(endpoint: Url, _auth: &AuthMethod) -> Result<ServiceInfo> {
-            if endpoint.port() == Some(5000) {
-                Ok(ServiceInfo {
-                    root_url: Url::parse(URL).unwrap(),
-                    current_version: Some(ApiVersion(1, 42)),
-                    minimum_version: Some(ApiVersion(1, 1)),
-                })
-            } else {
-                Err(Error::new(ErrorKind::EndpointNotFound, String::new()))
-            }
-        }
-
-        fn api_version_headers(version: ApiVersion) -> Option<Headers> {
-            if version >= ApiVersion(1, 1) && version <= ApiVersion(1, 42) {
-                Some(Headers::new())
-            } else {
-                None
-            }
-        }
-    }
 }
